use crate::config::dump::DumpConfig;
use serde::{Deserialize, Serialize};
use sysinfo::{Pid, System};
use tabled::{settings::Style, Table, Tabled};

const DEFAULT_ID: &str = "0";
const DEFAULT_NAMESPACE: &str = "default";
const DEFAULT_VERSION: &str = "N/A";
const DEFAULT_USER: &str = "N/A";

#[derive(Serialize, Deserialize)]
pub struct PmrProcess {
    pub pmr_id: u32,
    pub pid: u32,
    pub name: String,
    pub namespace: String,
    pub program: String,
    pub args: Vec<String>,
    pub status: String,
    pub restarts: u32,
}

#[derive(Tabled)]
struct ProcessInfo {
    id: String,
    name: String,
    namespace: String,
    version: String,
    pid: String,
    uptime: String,
    restarts: String,
    status: String,
    cpu: String,
    mem: String,
    user: String,
}

fn time_to_readable(seconds: u64) -> String {
    if seconds == 0 {
        return "0s".to_string();
    }

    let (days, hours, minutes, secs) = (
        seconds / 86400,
        (seconds % 86400) / 3600,
        (seconds % 3600) / 60,
        seconds % 60,
    );

    let parts = vec![
        if days > 0 {
            format!("{}d", days)
        } else {
            String::new()
        },
        if hours > 0 {
            format!("{}h", hours)
        } else {
            String::new()
        },
        if minutes > 0 {
            format!("{}m", minutes)
        } else {
            String::new()
        },
        if days == 0 && hours == 0 && secs > 0 {
            format!("{}s", secs)
        } else {
            String::new()
        },
    ];

    parts
        .into_iter()
        .filter(|s| !s.is_empty())
        .collect::<Vec<_>>()
        .join(" ")
}

pub fn read_pmr_processes() -> Vec<PmrProcess> {
    let dump_config = DumpConfig::get_instance();
    dump_config
        .list_processes()
        .map(|v| {
            v.into_iter()
                .map(|p| PmrProcess {
                    pmr_id: p.pmr_id,
                    pid: p.pid,
                    name: p.name,
                    namespace: p.namespace,
                    program: p.program,
                    args: p.args,
                    status: p.status,
                    restarts: p.restarts,
                })
                .collect()
        })
        .unwrap_or_else(|e| {
            eprintln!("Failed to read processes: {}", e);
            Vec::new()
        })
}

pub fn list_processes(system: bool) {
    let sys = System::new_all();

    let processes: Vec<ProcessInfo> = if system {
        sys.processes()
            .iter()
            .map(|(pid, process)| create_process_info(*pid, process, &sys))
            .collect()
    } else {
        read_pmr_processes()
            .into_iter()
            .map(|p| create_pmr_process_info(&p, &sys))
            .collect()
    };

    let mut table = Table::new(processes);
    if system {
        table.with(Style::modern());
    } else {
        table.with(Style::rounded());
    }
    println!("{}", table);
}

fn create_process_info(pid: Pid, process: &sysinfo::Process, sys: &System) -> ProcessInfo {
    let (status, sys_proc) = if process.name().to_str() == Some("system") {
        (process.status().to_string(), None)
    } else {
        (String::new(), sys.process(pid))
    };

    let (cpu, mem) = if let Some(proc) = sys_proc.or(Some(process)) {
        (
            format!("{:.1}%", proc.cpu_usage()),
            format!("{:.1} MB", proc.memory() as f64 / 1024.0 / 1024.0),
        )
    } else {
        ("0%".to_string(), "0 MB".to_string())
    };

    ProcessInfo {
        id: DEFAULT_ID.to_string(),
        name: process.name().to_str().unwrap_or("unknown").to_string(),
        namespace: DEFAULT_NAMESPACE.to_string(),
        version: DEFAULT_VERSION.to_string(),
        pid: pid.as_u32().to_string(),
        uptime: time_to_readable(process.run_time()),
        restarts: "0".to_string(),
        status,
        cpu,
        mem,
        user: DEFAULT_USER.to_string(),
    }
}

fn create_pmr_process_info(p: &PmrProcess, sys: &System) -> ProcessInfo {
    let (status, run_time, sys_proc) = if p.pid > 0 {
        let pid = Pid::from(p.pid as usize);
        match sys.process(pid) {
            Some(proc) => ("running".to_string(), proc.run_time(), Some(proc)),
            None => ("stopped".to_string(), 0, None),
        }
    } else {
        ("stopped".to_string(), 0, None)
    };

    let (cpu, mem) = sys_proc
        .map(|proc| {
            (
                format!("{:.1}%", proc.cpu_usage()),
                format!("{:.1} MB", proc.memory() as f64 / 1024.0 / 1024.0),
            )
        })
        .unwrap_or_else(|| ("0%".to_string(), "0 MB".to_string()));

    ProcessInfo {
        id: p.pmr_id.to_string(),
        name: p.name.clone(),
        namespace: p.namespace.clone(),
        version: DEFAULT_VERSION.to_string(),
        pid: p.pid.to_string(),
        uptime: time_to_readable(run_time),
        restarts: p.restarts.to_string(),
        status,
        cpu,
        mem,
        user: DEFAULT_USER.to_string(),
    }
}
